PROGRAM ReePlay3;

{$R ReePlay3.RES}
{$D ReePlay 1.2}

USES OWindows, ODialogs, WinProcs, WinTypes, BWCC, MMSystem, Strings, CDAudio, TrackWin;

CONST
  AppName: pChar = 'Player';
  (* Button id codes *)
  id_SkipBack   = 101;
  id_SkipForw   = 102;
  id_SearchBack = 103;
  id_SearchForw = 104;
  id_Pause      = 105;
  id_Stop       = 106;
  id_Play       = 107;
  id_Volume     = 108;
  id_Info       = 109;
  id_Tracks     = 110;
  id_MMSS       = 111;
  id_TimeMode   = 112;
  id_Shuffle    = 113;
  (* System Menu Codes *)
  sc_DefineName = 201;
  sc_About      = 202;
  (* CD-Identification Edit *)
  id_TheEdit    = 113;
  (* jump length in seconds *)
  SearchLen     =  10;
  (* Parameters for LEDs *)
  Obe           = 5;
  Lings         = 5;
  LedWidth      = 13;
  LedHeight     = 23;

TYPE
  TDlgApp = OBJECT(tApplication)
    PROCEDURE InitMainWindow; VIRTUAL;
  END;

TYPE
  PReePlay = ^TReePlay;
  TReePlay = OBJECT(tDlgWindow)
    IsPlaying, IsPaused: BOOLEAN;
    Track, Min, Sec: BYTE;
    CDName: ARRAY [0..59] OF CHAR;
    CONSTRUCTOR Init(aParent: pWindowsObject; aTitle: pChar);
    FUNCTION GetClassName: pChar; VIRTUAL;
    PROCEDURE GetWindowClass (VAR aWndClass: tWndClass); VIRTUAL;
    PROCEDURE SetupWindow; VIRTUAL;
    PROCEDURE Destroy; VIRTUAL;

    (* message response methods *)
    PROCEDURE WMTimer (VAR Msg: tMessage); VIRTUAL wm_First + wm_Timer;
    PROCEDURE WMSysCommand (VAR Msg: tMessage); VIRTUAL wm_First + wm_SysCommand;

    (* button response methods *)
    PROCEDURE SkipBack (VAR Msg: tMessage); VIRTUAL id_First + id_SkipBack;
    PROCEDURE SkipForw (VAR Msg: tMessage); VIRTUAL id_First + id_SkipForw;
    PROCEDURE SearchBack (VAR Msg: tMessage); VIRTUAL id_First + id_SearchBack;
    PROCEDURE SearchForw (VAR Msg: tMessage); VIRTUAL id_First + id_SearchForw;
    PROCEDURE Pause (VAR Msg: tMessage); VIRTUAL id_First + id_Pause;
    PROCEDURE Stop (VAR Msg: tMessage); VIRTUAL id_First + id_Stop;
    PROCEDURE Play (VAR Msg: tMessage); VIRTUAL id_First + id_Play;
    PROCEDURE Volume (VAR Msg: tMessage); VIRTUAL id_First + id_Volume;
    PROCEDURE Info (VAR Msg: tMessage); VIRTUAL id_First + id_Info;

    PROCEDURE SetTMS (ATrack, AMin, ASec: BYTE);
    PROCEDURE SetTrack (HowMany: INTEGER);
    PROCEDURE SetMMSS (AMinute, ASecond: WORD);
  END;

CONSTRUCTOR TReePlay.Init(aParent: pWindowsObject; aTitle: pChar);
BEGIN
  tDlgWindow.Init(aParent, aTitle);
  IsPlaying := FALSE;
  IsPaused := FALSE;
  StrPCopy (CDName, '[Unknown CD]');
END;

FUNCTION TReePlay.GetClassName: pChar;
BEGIN
  GetClassName := 'BorDlg_CD';
END;

PROCEDURE TReePlay.GetWindowClass (VAR aWndClass: tWndClass);
BEGIN
  tDlgWindow.GetWindowClass (aWndClass);
  aWndClass.lpfnWndProc := @BWCCDefDlgProc;
  aWndClass.hIcon := LoadIcon (hInstance, 'Grapevine');
END;

PROCEDURE TReePlay.SetupWindow;
VAR
  SysMenu: hMenu;
BEGIN
  tDlgWindow.SetupWindow;
  SetTimer (hWindow, 1, 500, NIL);
  SysMenu := GetSystemMenu(hWindow, FALSE);
  RemoveMenu (SysMenu, sc_Size, mf_ByCommand);
  RemoveMenu (SysMenu, sc_Restore, mf_ByCommand);
  AppendMenu (SysMenu, mf_Separator, 0, '');
  AppendMenu (SysMenu, mf_Enabled, sc_About, 'About...');
  AppendMenu (SysMenu, mf_Separator, 0, NIL);
  AppendMenu (SysMenu, mf_String, sc_DefineName, 'Identify CD');
END;

PROCEDURE TReePlay.Destroy;
BEGIN
  tDlgWindow.Destroy;
  (* Soll nach dem Beenden der Applikation die CD weitergespielt werden, *)
  (* kommentieren Sie die folgende Zeile aus:                            *)
  cdStop;
  KillTimer (hWindow, 1);
END;

PROCEDURE TReePlay.SetTMS (ATrack, AMin, ASec: BYTE);
BEGIN
  Track := ATrack;
  Min := AMin;
  Sec := ASec;
END;

PROCEDURE TReePlay.WMTimer (VAR Msg: tMessage);
VAR
  Frame: BYTE;
BEGIN
  IF IsPlaying
    THEN GetTMSF (Track, Min, Sec, Frame)
    ELSE SetTMS (0, 0, 0);
  SetTrack (Track);
  SetMMSS (Min, Sec);
END;

PROCEDURE TReePlay.WMSysCommand (VAR Msg: tMessage);
VAR
  Dialog: pDialog;
  P: pWindowsObject;
BEGIN
  CASE Msg.wParam OF
    sc_DefineName: BEGIN
      Dialog := New (pDialog, Init(@Self, pChar('ArtistTitle')));
      Dialog^.TransferBuffer := @CDName;
      P := New (pEdit, InitResource(Dialog, id_TheEdit, 60));
      Application^.ExecDialog (Dialog);
    END;
    sc_About: BEGIN
      Dialog := New (pDialog, Init(@Self, 'AboutBox'));
      sndPlaySound ('g:\zebra\andi\cdplay\reeplay3\KRAFT3.WAV', snd_Async);
      Application^.ExecDialog (Dialog);
    END;
  END;
  DefWndProc (Msg);
END;

PROCEDURE TReePlay.SkipBack (VAR Msg: tMessage);
BEGIN
  IF (Track > 1) THEN cdPlay (Track-1, HowManyTracks)
                 ELSE cdPlay (1, HowManyTracks);
END;

PROCEDURE TReePlay.SkipForw (VAR Msg: tMessage);
BEGIN
  IF (Track < HowManyTracks) THEN cdPlay (Track+1, HowManyTracks)
                             ELSE cdPlay (HowManyTracks, HowManyTracks+1);
END;

PROCEDURE TReePlay.SearchBack (VAR Msg: tMessage);
BEGIN
  cdStop;
  IF (Sec < SearchLen) THEN BEGIN
    Sec := Sec + 60 - SearchLen;
    Dec (Min);
  END;
  cdGoto (Track, Min, Sec-SearchLen);
END;

PROCEDURE TReePlay.SearchForw (VAR Msg: tMessage);
BEGIN
  cdStop;
  IF (Sec > 60-SearchLen) THEN BEGIN
    Inc (Min);
    Sec := Sec - 60 + SearchLen;
  END;
  cdGoto (Track, Min, Sec+SearchLen);
END;

PROCEDURE TReePlay.Pause (VAR Msg: tMessage);
VAR
  ReturnCode: LONGINT;
BEGIN
  IF IsPaused THEN BEGIN
    cdGoto (Track, Min, Sec);
    IsPaused := FALSE;
  END ELSE BEGIN
    cdStop;
    IsPaused := TRUE;
  END;
END;

PROCEDURE TReePlay.Stop (VAR Msg: tMessage);
VAR
  ReturnCode: LONGINT;
BEGIN
  IF IsPlaying THEN BEGIN
    ReturnCode := cdStop;
    IF (ReturnCode <> 0) THEN ShowError (ReturnCode);
    ReturnCode := cdClose;
    IsPlaying := FALSE;
  END;
END;

PROCEDURE TReePlay.SetMMSS (AMinute, ASecond: WORD);
VAR
  Args: ARRAY [0..1] OF WORD;
  DiSi, MemDC: hDC;
  DigitInfo: tBitmap;
  Digit, OldBitmap: hBitmap;
  Result: WORD;
BEGIN
  Args[0] := Min;
  Args[1] := Sec;
  DiSi := GetDC (GetDlgItem(hWindow, id_MMSS));
  Digit := LoadBitmap(hInstance, pChar ( (Args[0] MOD 10) + 300) );
  MemDC := CreateCompatibleDC (DiSi);
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth*2), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);

  Digit := LoadBitmap (hInstance, pChar ( (Args[0] DIV 10) + 300) );
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth * 1), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);

  Digit := LoadBitmap (hInstance, 'MinSecSep' );
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth * 3), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);

  Digit := LoadBitmap (hInstance, pChar ( (Args[1] MOD 10) + 300) );
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth * 5), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);

  Digit := LoadBitmap (hInstance, pChar ( (Args[1] DIV 10) + 300) );
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth * 4), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);
  DeleteDC (MemDC);
  Result := ReleaseDC (hWindow, DiSi);
END;

PROCEDURE TReePlay.SetTrack (HowMany: INTEGER);
VAR
  DiSi, MemDC: hDC;
  DigitInfo: tBitmap;
  Digit, OldBitmap: hBitmap;
  Result: WORD;
BEGIN
  DiSi := GetDC (GetDlgItem(hWindow, id_Tracks));
  Digit := LoadBitmap(hInstance, pChar ( (HowMany MOD 10) + 300) );
  MemDC := CreateCompatibleDC (DiSi);
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth*3), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);

  Digit := LoadBitmap (hInstance, pChar ( (HowMany DIV 10) + 300) );
  OldBitmap := SelectObject (MemDC, Digit);
  BitBlt (DiSi, Lings + (LedWidth * 2), Obe, LedWidth, LedHeight, MemDC, 0, 0, SrcCopy);
  SelectObject (MemDC, OldBitmap);
  DeleteObject (Digit);
  DeleteDC (MemDC);
  Result := ReleaseDC (hWindow, DiSi);

END;

PROCEDURE TReePlay.Play (VAR Msg: tMessage);
VAR
  ReturnCode: LONGINT;
BEGIN
  IF NOT IsPlaying THEN BEGIN
    ReturnCode := cdOpen;
    ReturnCode := cdPlay (1, HowManyTracks);
    SetTrack (HowManyTracks);
    IsPlaying := TRUE;
  END;
END;

PROCEDURE TReePlay.Volume (VAR Msg: tMessage);
VAR
  What2Exec, szDefault: ARRAY [0..79] OF CHAR;
  WinPath: ARRAY [0..99] OF CHAR;
  RetCode, wLength: WORD;
BEGIN
  (* Version zum Holen des Volume Control Pfades aus einer INI-Datei *)
  {
  GetWindowsDirectory (WinPath, 144);
  StrCat (WinPath, '\reeplay.ini');
  }
  StrPCopy(WinPath,'reeplay.ini');
  RetCode := GetPrivateProfileString ('ReePlay', 'Volume',
     szDefault, What2Exec, wLength, WinPath);
  IF (wLength > 0) THEN
  BEGIN
    RetCode := WinExec (What2Exec, sw_Normal);
    IF (RetCode < 32) THEN MessageBox (0,
      'Volume (SBPro Mixer) couldn''t be launched.'#13#10'Please check path name in stringtable resource.',
      'Problem', mb_OK);
  END;
END;

PROCEDURE TReePlay.Info (VAR Msg: tMessage);
BEGIN
  IF (TrackWindow = NIL) AND IsPlaying THEN
  BEGIN
    TrackWindow := New (PTrackWindow, Init (CDName));
    IF (Application^.MakeWindow (TrackWindow) <> NIL)
      THEN TrackWindow^.Show (sw_Show);
  END;
END;

PROCEDURE TDlgApp.InitMainWindow;
BEGIN
  MainWindow := New(PReePlay,Init(NIL,MakeIntResource('CDPlayer')));
END;

VAR
  MyDlgApp: TDlgApp;

BEGIN
  MyDlgApp.Init(AppName);
  MyDlgApp.Run;
  MyDlgApp.Done;
END.
